@@ -1,6 +1,6 @@
 // Reference-counted versatile string base -*- C++ -*-
 
-// Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+// Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -37,11 +37,11 @@
 #define _RC_STRING_BASE_H 1
 
 #include <ext/atomicity.h>
+#include <bits/stl_iterator_base_funcs.h>
 
 _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
 
   /**
-   *  @if maint
    *  Documentation?  What's that?
    *  Nathan Myers <ncm@cantrip.org>.
    *
@@ -81,7 +81,6 @@ _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
    *
    *  All but the last paragraph is considered pretty conventional
    *  for a C++ string implementation.
-   *  @endif
   */
  template<typename _CharT, typename _Traits, typename _Alloc>
     class __rc_string_base
@@ -179,7 +178,7 @@ _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
       // (NB: last two terms for rounding reasons, see _M_create below)
       // Solving for m:
       // m = ((npos - 2 * sizeof(_Rep) + 1) / sizeof(_CharT)) - 1
-      // In addition, this implementation halfs this amount.
+      // In addition, this implementation halves this amount.
       enum { _S_max_size = (((static_cast<size_type>(-1) - 2 * sizeof(_Rep)
 			      + 1) / sizeof(_CharT)) - 1) / 2 };
 
@@ -231,12 +230,13 @@ _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
           return _S_construct(__beg, __end, __a, _Tag());
 	}
 
-      template<typename _InIterator>
+      // _GLIBCXX_RESOLVE_LIB_DEFECTS
+      // 438. Ambiguity in the "do the right thing" clause
+      template<typename _Integer>
         static _CharT*
-        _S_construct_aux(_InIterator __beg, _InIterator __end,
+        _S_construct_aux(_Integer __beg, _Integer __end,
 			 const _Alloc& __a, std::__true_type)
-	{ return _S_construct(static_cast<size_type>(__beg),
-			      static_cast<value_type>(__end), __a); }
+	{ return _S_construct(static_cast<size_type>(__beg), __end, __a); }
 
       template<typename _InIterator>
         static _CharT*
@@ -299,12 +299,18 @@ _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
       { _M_rep()->_M_set_length(__n); }
 
       __rc_string_base()
-      : _M_dataplus(_Alloc(), _S_empty_rep._M_refcopy()) { }
+      : _M_dataplus(_S_empty_rep._M_refcopy()) { }
 
       __rc_string_base(const _Alloc& __a);
 
       __rc_string_base(const __rc_string_base& __rcs);
 
+#ifdef __GXX_EXPERIMENTAL_CXX0X__
+      __rc_string_base(__rc_string_base&& __rcs)
+      : _M_dataplus(__rcs._M_get_allocator(), __rcs._M_data())
+      { __rcs._M_data(_S_empty_rep._M_refcopy()); }      
+#endif
+
       __rc_string_base(size_type __n, _CharT __c, const _Alloc& __a);
 
       template<typename _InputIterator>
@@ -545,7 +551,7 @@ _GLIBCXX_BEGIN_NAMESPACE(__gnu_cxx)
 	  return _S_empty_rep._M_refcopy();
 
 	// NB: Not required, but considered best practice.
-	if (__builtin_expect(_S_is_null_pointer(__beg) && __beg != __end, 0))
+	if (__builtin_expect(__is_null_pointer(__beg) && __beg != __end, 0))
 	  std::__throw_logic_error(__N("__rc_string_base::"
 				       "_S_construct NULL not valid"));
 
