@@ -1,6 +1,6 @@
 // 2005-06-07 Benjamin Kosnik
 
-// Copyright (C) 2005
+// Copyright (C) 2005, 2006
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -16,7 +16,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 #include <sstream>
@@ -27,7 +27,7 @@ double
 test_stringstream()
 {
   double result;
-  char* source = "1918"; 
+  const char* source = "1918"; 
   std::stringstream s;
   s << source;
 
